pub mod linux;

lazy_static::lazy_static! {
    pub static ref MULTICAST_ADDRESS : std::net::SocketAddrV4 =
        std::net::SocketAddrV4::new(
            std::net::Ipv4Addr::new(239, 0, 2, 35),
            3535
        );
}

pub struct Packet {
    pub(crate) addr: Option<std::net::SocketAddr>,
    pub(crate) data: Option<Vec<u8>>,
    pub(crate) channel: Option<std::rc::Rc<dyn IOChannel>>,
    pub msg: Option<crate::msg::Message>,
}

pub(crate) trait IOChannel {
    fn send_packet(&self, packet: &mut Packet);
}

pub(crate) trait IOFeedback {
    fn packet(&self, packet: Packet);
    fn minor_tick(&self);
    fn major_tick(&self);
}

pub(crate) trait FleckIO {
    fn poll<'a>(&self, f: &dyn IOFeedback);

    fn local(&self) -> std::rc::Rc<dyn IOChannel>;
    fn global(&self) -> std::rc::Rc<dyn IOChannel>;
}

pub(crate) trait IOHandler {
    fn minor(&self);
}

pub(crate) fn platform() -> std::rc::Rc<dyn FleckIO> {
    std::rc::Rc::new(linux::LinuxIO::default())
}

