use crate::io::{FleckIO, Packet};

pub(crate) struct SendPacket {
    io: std::rc::Rc<dyn FleckIO>,
}

impl SendPacket {
    pub(crate) fn new(io: std::rc::Rc<dyn FleckIO>) -> Self {
        Self { io }
    }
}

impl crate::FleckService for SendPacket {
    fn process_outgoing(&self, packet: &mut crate::io::Packet) {
        // use default channel if not specified
        if packet.channel.is_none() {
            packet.channel = Some(self.io.global());
        }

        // serialize if needed
        match (packet.data.as_ref().is_some(), packet.msg.take()) {
            (false, Some(msg)) => {
                packet.data = Some(bincode::serialize(&msg).expect("failed to serialize message"));
            },
            _ => {},
        }

        match &mut packet.channel {
            Some(ch) => ch.clone().send_packet(packet),
            _ => {
                println!("tried to send packet with no channel?");
            }
        }
    }
}

pub(crate) struct LocalDiscovery {
    io: std::rc::Rc<dyn FleckIO>,
}

impl LocalDiscovery {
    pub(crate) fn new(io: std::rc::Rc<dyn FleckIO>) -> Self {
        Self { io }
    }
}

impl crate::FleckService for LocalDiscovery {
    fn process_major_tick(&self, si: &dyn super::ServiceInterface) {
        si.send(Packet {
            addr: Some(crate::io::MULTICAST_ADDRESS.clone().into()),
            data: None,
            channel: Some(self.io.local()),
            msg: Some(crate::msg::Message::build(crate::msg::HeaderType::Hello)),
        });
    }
}
